<?php
/* Written by Nic Roets. Placed in the public domain. */
class GoogleEarthAlbumController extends GalleryController {
    function handleRequest($form) {
        if (!isset($form['tmp_name'][1])) {
            return array (GalleryCoreApi::error(ERROR_BAD_PARAMETER, __FILE__, __LINE__, 'Tracklog file is required'), null);
        }
        if (preg_match('/^([-+0-9]+):([0-9]+)$/', $form['zone'], $regs) != 1) {
            return array (GalleryCoreApi::error(ERROR_BAD_PARAMETER, __FILE__, __LINE__, 'Invalid timezone'), null);
        }
        $itemId = GalleryUtilities::getRequestVariables('itemId');
        list ($ret, $item) = GalleryCoreApi::loadEntitiesById($itemId);
        if ($ret) {
             return $ret->wrap(__FILE__, __LINE__);
        }
        $track = fopen ($form['tmp_name'][1], 'r');
        $next = array ();
        $next[2] = 0;
        $next[4] = 0;
        $next[20] = -1e+10;
        $heading = 0;

        list ($ret, $childIds) = GalleryCoreApi::fetchChildItemIds($item);
        if ($ret) {
             return $ret->wrap(__FILE__, __LINE__);
        }
        $childItems = array ();
        if (!empty($childIds)) {
            /* Load the children */
            list ($ret, $childItems) = GalleryCoreApi::loadEntitiesById($childIds);
            if ($ret) {
                 return $ret->wrap(__FILE__, __LINE__);
            }
        }
        GalleryCoreApi::requireOnce ('modules/exif/classes/ExifHelper.class');
        GalleryCoreApi::requireOnce ('modules/map/classes/mapHelper.class');

        $offset = $regs[1]*3600 + ($regs[1] > 0 ? $regs[2] : -$regs[2])*60  -
          (int)(date('O')/100)*3600 - (date('O') > 0 ? 60 : -60) * (date('O')%100);
        //echo "offset=$offset <br>";
        global $gallery;
        $ugen=$gallery->getUrlGenerator();
        $urlGenerator = new GalleryUrlGenerator();
        $urlGenerator->init($ugen->_baseFile, $ugen->_embedPath,
        $ugen->_relativeG2Path, $ugen->_embedSessionString /* Leave blank ? */);
        $photos = array();

        foreach ($childItems as $child) {
            if (!GalleryUtilities::isA($child, 'GalleryDataItem')) continue;
            list($ret, $path) = $child->fetchPath();
            $timestamp = ExifHelper::getOriginationTimestamp($path) - $offset;
            $photos[$timestamp] = $child;
            $offset -= 0.01; /* Just to prevent duplicate timstamps */
        }
        ksort ($photos);
        header ('Content-Type: application/vnd.google-earth.kml+xml');
        echo '<?xml version="1.0" encoding="UTF-8"?>
        <kml xmlns="http://earth.google.com/kml/2.0">
        <Folder>
        <name>'. $item->getTitle() .'</name>';
        foreach ($photos as $timestamp => $child) {
            list ($ret, $source) = GalleryCoreApi::fetchThumbnailsByItemIds(array($child->getId()));
            if ($ret) {
                 return $ret->wrap(__FILE__, __LINE__);
            }

            global $gallery;
            $gurl=&$gallery->getUrlGenerator();
            $thisId=$child->getId();
    
            // Select items that have GPS coordinates filled out
            $query='
                SELECT [GallerymapMap::itemId],
                    [GallerymapMap::value]
                FROM [GallerymapMap]
                WHERE [GallerymapMap::field] = \'GPS\'
                AND [GallerymapMap::itemId] = \''.$thisId.'\'
            ';
            list($ret, $results) = $gallery->search($query, array(), array());
    
            //If there is more than 0 Item :-) continue on ...
            if (!$ret and $results->resultCount() > 0) {
    
              while($result=$results->nextResult()) {
      
                  $posxy = explode(",", $result[1]);
                  if (count($posxy) == 2) {
                      $lon=(float)$posxy[0];
                      $lat=(float)$posxy[1];
                  }
      
                  if ($lon != 0 and $lat != 0 and substr($result[0],0,1) !="G") {
      
                    $theid=$result[0];
                    list($ret, $thumbs)=GalleryCoreApi::fetchThumbnailsByItemIds(array($result[0]));
                    
                    /* SJK resizes */
                    $kkkId=$result[0];
                    list ($ret, $resizeTable) = GalleryCoreApi::fetchResizesByItemIds(array($kkkId));
                    if (isset($resizeTable[$kkkId][0])) {
        
                       $resizeId = $resizeTable[$kkkId][0]->getId();
        
                    }
                    else
                    {
                      $resizeId = $id;
                      
                    }
                    /* SJK resizes */
        
                    $thumbId=$thumbs[$theid]->getId();;
                    $thumbWidth=$thumbs[$theid]->getWidth();
                    $thumbHeight=$thumbs[$theid]->getHeight();
        
        
                    $dthumbLink=$urlGenerator->generateUrl(array('view'=>'core.DownloadItem','itemId'=>$thumbId), array('forceFullUrl' => true));
                    $hsitemLink=$urlGenerator->generateUrl(array('view'=>'core.ShowItem','itemId'=>$result[0]), array('forceFullUrl' => true));
        
                    $hditemLink=$urlGenerator->generateUrl(array('view'=>'core.DownloadItem','itemId'=>$resizeId), array('forceFullUrl' => true));

                    $coord = $lon . ',' . $lat;
      
                  }
              }
            }
    
            /* Output $coord to Google Earth */
    
            if ($hsitemLink != "" and $dthumbLink != "") {
                echo '<Placemark><name>' . $child->getTitle() . "</name>\r\n";
                echo '<description><![CDATA[<a href="'.$hsitemLink;
                echo "\">\r\n<img src=\"".$dthumbLink;
                echo "\"></a>]]></description>\r\n";
                echo '<LookAt><longitude>' . $lon . '</longitude><latitude>' . $lat;
                echo '</latitude><range>3000</range><tilt>45</tilt><heading>';
                echo $heading . "</heading></LookAt>\r\n";
                echo '<styleUrl>root://styleMaps#default+nicon=0x307+hicon=0x317</styleUrl>';
                echo '<Style><IconStyle><Icon><href>root://icons/palette-4.png</href>';
                echo '<x>192</x><y>96</y><w>32</w><h>32</h></Icon></IconStyle></Style>';
                echo "\r\n<Point><extrude>0</extrude>\r\n";
                echo "<tessellate>0</tessellate>\r\n";
                echo '<altitudeMode>clampToGround</altitudeMode>';
                echo "\r\n<coordinates>" . $coord;
                echo ",0</coordinates>\r\n</Point>\r\n</Placemark>\r\n\r\n";
            }
        }
        echo "\r\n</Folder></kml>\r\n";
        exit;
        return array(null, array ('status' => array ('changed', 1), 'return' => 1, 'redirect' => '', error => array()));
    }
}

class GoogleEarthAlbumView extends GalleryView {
    function loadTemplate (&$template, &$form) {
        global $gallery;
        list ($ret, $item) = $this->_getItem();

        $form['formName'] = 'GoogleEarthAlbum';
        $GoogleEarthAlbum['enctype'] = 'multipart/form-data';
        $GoogleEarthAlbum['zone'] = '+0:00';
        preg_match('/^([-+0-9]+)([0-9][0-9])$/', date('O'), $regs);
        $form['zone'] = $regs[1] . ':' . $regs[2];
        $GoogleEarthAlbum['track'] = '';
        $GoogleEarthAlbum['itemId'] = $item->getId();
        $template->setVariable('GoogleEarthAlbum', $GoogleEarthAlbum);
        $template->setVariable('controller', 'map.GoogleEarthAlbum');
        return array(null,
                     array('body' =>
                        'modules/map/templates/GoogleEarthAlbum.tpl'));
    }
    function getTitle () {
        list ($ret, $module) = GalleryCoreApi::loadPlugin('module', 'core');


        return array(null, $gallery->i18n('Google Earth Album Export'));
    }
}
?>
